;===============================================================================
;Famicom Disk System Program Template v0.92a
;FDS Disk Image (fwNES .fds format)
;By Pokun
;
;Assembles in ASM6
;===============================================================================

.segment "CODE"

;General FDS info:
;PRG ROM : 8 KB BIOS at $E000-$FFFF
;PRG RAM: 32 KB at $6000-$DFFF
;CHR capacity: 8 KB RAM
;Storage capacity : ~64 KB per disk side
;Nametable mirroring: Controlled by mapper
;Subject to bus conflicts: No
;Games are stored on one or multiple disk sides (every disk has 2 sides).
;The FDS BIOS is used to load data from disks to PRG RAM or VRAM,
;and games can execute from there. 

.include "constants.asm"

;===============================================================================
; Variables
;===============================================================================
;Zero Page ($0000-$00FF)
  .segment "ZP"

;Page 1 ($0100-$01FF)
;(Stack)
  .segment "STACK" ; i guess?

;Page 2 ($0200-$02FF)
;(Shadow OAM)

;Page 3-7 ($0300-$07FF)
  .segment "RAM"

;-------------------------------------------------------------------------------
;  .org $0000
.segment "CODE" ;start assembling

;===============================================================================
; fwNES FDS header
;===============================================================================
;16 byte header for the .fds Quick Disk format.
;This header is optional but some emulators may require it. FDSStick and some
;emulators like FCEUX doesn't require it since the number of disk sides may
;be figured out simply by looking at the file size of the image file.

.segment "HEADER"

  .byte "FDS", $1A ;identification of the fwNES FDS header
  .byte SIDE_COUNT ;number of disk sides

;HeaderOffset = $ ;add this to compensate for the .fds header when calculating
                 ;the disk image size.


;Note about disk structure and blocks
;Each disk side must be structured into blocks as follows:
;First a disk header block and a file amount block, then one file header block
;and one file data block for each file on the disk. Finally the disk sides ends
;with an end of disk side indicator (not sure if needed).
;Each block has a block code:
;$01 = Disk Header Block
;$02 = File Amount Block
;$03 = File Header Block
;$04 = File Data Block
;$FF = End of Disk Side Indicator

;From the last file, fill the side with zeroes so that the disk side gets
;exactly 65500 bytes (excluding the fwNES header).

;On real disks there are also gaps and CRCs between each block. These are not
;present in the fwNES FDS image format. FDSStick and similar devices
;calculates and adds them automatically. But they must be considered when
;calculating true disk capacity. The actual disc capacity in bytes is:
;65500 - 28300/8 - (2*N + 1)*(16 + 976)/8
;N = number of files

;===============================================================================
; Disk Side 1 (65500 byte)
;===============================================================================

;Disk Header Block
  .segment "CODE"
  .byte $01 ;Block code (1 = disk header block)
  .byte "*NINTENDO-HVC*" ;Used by BIOS to verify legitimate image.
  ;If the string is wrong the disk will fail to boot.
  .byte $00 ;Maker code (similar codes as in Game Boy headers)
  ;$00 is unlicenced, $01 is Nintendo $08 is Capcom...
  .byte "HMB" ;3-letter ASCII code for title (e.g. LNK for Zelda no Densetsu)
  .byte " " ;Game type
  ;$20 = " " — Normal disk
  ;$45 = "E" — Event (e.g. Japanese national DiskFax tournaments)
  ;$52 = "R" — Reduction in price via advertising
  .byte REVISION ;Revision number (starts at $00, increments per revision)
  .byte $00 ;Side number
  ;$00 = Side A, $01 = Side B. Single-sided disks use $00
  .byte $00 ;Disk number (first disk is $00, second is $01, etc.)
  .byte $00 ;Disk type
  ;$00 = FMC ("normal card"), $01 = FSC ("card with shutter")
  .byte $00 ;Unknown
  .byte $0F ;Boot read file code (file code/number to load upon boot)
  .byte $FF,$FF,$FF,$FF,$FF ;Unknown
  .byte $00,$00,$00 ;Manufacturing date
  ;Stored in BCD, subtract the Shouwa starting year 1925 from the year
  ;Example: 1988-11-28 becomes $63 $11 $28
  .byte $49 ;Country code ($49 = Japan)
  .byte $61 ;Unknown
  .byte $00 ;Unknown
  .byte $00,$02 ;Unknown
  .byte $00,$00,$00,$00,$00 ;Unknown
  .byte $00,$00,$00 ;"Rewritten disk" date (supposedly by the Disk Writer)
  .byte $00 ;Unknown
  .byte $80 ;Unknown
  .byte $00,$00 ;Disk Writer serial number
  .byte $07 ;Unknown
  .byte $00 ;Disk rewrite count (value in BCD, $00 = Retail Disk)
  .byte $00 ;Actual disk side ($00 = Side A, $01 = Side B)
  .byte $00 ;Unknown
  .byte $00 ;Debug version or price
  ;.byte $00,$00 ;CRC - Not used in the .fds file format

;Disk sides note
;If the FDS is started with a disk whose side number and disk number
;aren't both $00, it will be prompted to insert the first disk side.
;However, some games make this number $00, even for the second disk
;to make it bootable too.

;-------------------------------------------------------------------------------
;File Amount block
  .byte $02 ;Block code (2 = file amount block)
  .byte FILE_COUNT1 ;Total number of files recorded on disk (side?)
  ;If more files exist on disk they will be considered hidden and the
  ;BIOS will ignore them. A loading routine on disk is required to load them.
  ;.byte $00,$00 ;CRC - Not used in the .fds file format

;File codes note
;All files with IDs smaller or equals to the boot read file code
;will be loaded when the game is booting.

;-------------------------------------------------------------------------------
;File 0 "KYODAKU-"

;File Description:
;The first file on the disk must be the "KYODAKU-" file (approval in Japanese)
;that contains the message that scrolls up on screen ("THIS PRODUCT
;IS MANUFACTERED AND SOLD BY NINTENDO...") at boot, and must match
;the same 224-byte text data in the BIOS at $ED37 or the disk won't boot.
;It's a nametable type of file that is loaded into PPU $2800.

.include "file0.asm"

;-------------------------------------------------------------------------------
;File 1 "MAIN-PRG"

;File Description:
;This file contains the main program and data. Program files should be loaded
;to PRAM ($6000~$DFFF) on the RAM Adapter.
;If the program is 16 kB or less it may be loaded to $A000 instead of $6000.

.incbin "file1.bin"

;-------------------------------------------------------------------------------
;File 2 "CHARA-00"

;File Description:
;This file contains BG CHR data (background graphics) that is to be loaded to
;the RAM Adapter.

;File Header Block
  .byte $03 ;Block code
  .byte $02 ;File Number
  .byte $02 ;File ID
  .byte "CHARA-00" ;File Name
  .word FDS_CRAM+$1000 ;Load location (CHR data goes to CRAM in RAM Adapter)
  .byte $00,$10 ;File Size
  .byte $01 ;File type (CHR)
  ;.byte $00,$00 ;CRC - Not used in the .fds file format

;File Data Block
  .byte $04 ;Block code
  .incbin "file2.chr" ;file data
  ;.byte $00,$00 ;CRC - Not used in the .fds file format

;-------------------------------------------------------------------------------
;File 3 "CHARA-01"

;File Description:
;This file contains OBJ CHR data (sprite graphics) that is to be loaded to
;the RAM Adapter.

;File Header Block
  .byte $03 ;Block code
  .byte $03 ;File Number
  .byte $03 ;File ID
  .byte "CHARA-01" ;File Name
  .word FDS_CRAM ;Load location
  .byte $00,$10 ;File Size
  .byte $01 ;File type (CHR)
  ;.byte $00,$00 ;CRC - Not used in the .fds file format

;File Data Block
  .byte $04 ;Block code
  .incbin "file3.chr" ;file data
  ;.byte $00,$00 ;CRC - Not used in the .fds file format
  
;-------------------------------------------------------------------------------
;End of Disk Side 1
  .byte $FF ;End of disk side indicator
  
;-------------------------------------------------------------------------------
;Next disk side goes here. A header for this side is needed but no KYODAKU-file
;unless it's bootable I think.
